import { User_ObjectValueDataType } from "@elowcode/robotdev-design-types"

export type String = string
export type Number = number
export type Boolean = boolean
export type Object = { [key: string]: any }
export type Array = any[]

export type BindId = string
export type DeclName = string
export type Call = [string, CallArg[]]
export type CallArg = string | number | boolean | any[] | { [key: string]: any } | Call

export type Path = (string | number)[]

// 变量获取和修改
export interface DeclState {
  get: (id: string | number, path?: Path) => any
  set: (id: string | number, path: Path | undefined, value: any, type?: 'var' | 'closure') => void
}

// 组件属性绑定
export interface BindState {
  set: (id: string, name: string, getValue: () => any) => void
}

// 组件事件绑定
export interface OnState {
  set: (id: string, name: string, func: (...args: any[]) => any) => void
}

// 钩子绑定相关定义
export interface HooksState {
  set: (name: string, func: (...args: any[]) => any) => void
}

// 属性获取
export interface PropsState {
  get: (name: string, path?: Path) => any
}

// 上下文相关定义
export interface CtxState {
  get: (key: string, path: Path) => any
}

// 页面参数
export interface ParamsState {
  get: (id: string, path: Path) => any
}

// 循环内变量获取
export interface EachState {
  get: (id: string, type: 'item' | 'index' | 'key', path?: Path) => any
}

// 插槽相关定义
export interface SlotState {
  get: (id: string, path: Path) => any
}

export interface Router {
  push: (url: string) => void
  replace: (url: string) => void
  blank: (url: string) => void
}

export type RouteConfig = { path: string; params: any; query: any }
export type Route = (config: RouteConfig) => string
export type Remote = (config: { [key: string]: any }, callback: (data: any) => void) => void
export type RemoteBusiness = (
  apiId: string,
  config?: {
    body?: any
    filter?: any
    order?: any
    pageLength?: number
  },
  success?: (res: any) => void,
  fail?: (error: any) => void
) => void

export type Request = (
  url: string,
  config: {
    method?: string
    query?: { [key: string]: string }
    data?: any
  },
  success: (data: any) => void,
  fail: (msg: string) => void
) => void

interface NodePropsState {
  get(nodeId: string, key: string, path?: Path): any
  set(nodeId: string, key: string, path: Path | undefined, value: any): void
}

interface ApiDataState {
  get(apiId: string, path?: Path): any
  set(apiId: string, path: Path | undefined, value: any): void
}

// 函数形参状态
export interface ArgsState {
  list: () => any[][]
  get: (level: number, index: number, path: Path) => any
  length: (level: number) => number
}
export interface PrivateCtx {
  args: ArgsState
  newArgs: (args: any[]) => PrivateCtx
}
export interface Ctx {
  global: GlobalCtx
  private: PrivateCtx
}
interface EventDataState {
  get(id: string, path?: Path): any
}

// 上下文方法集
export type GlobalCtx = {
  getModuleDefineData: (data: any) => User_ObjectValueDataType[]
  nodeProps: NodePropsState
  apiData: ApiDataState
  params?: ParamsState
  eventArgs: EventDataState
  remoteBusiness: RemoteBusiness
  decl: DeclState
  props: PropsState
  each: EachState
  slot: SlotState
  ctx: CtxState
  emit?: (name: string, args: any[]) => void
  router?: Router
  route?: Route
  remote?: Remote
  request?: Request
}

// 入参配置
export type StmtParseApi = {
  getModuleDefineData?: (data: any) => User_ObjectValueDataType[]
  nodeProps?: NodePropsState
  apiData?: ApiDataState
  params?: ParamsState
  eventArgs?: EventDataState
  remoteBusiness?: RemoteBusiness
  decl?: DeclState
  each?: EachState
  slot?: SlotState
  props?: PropsState
  ctx?: CtxState
  emit?: (name: string, args: any[]) => void
  router?: Router
  route?: Route
  remote?: Remote
  request?: Request
}

// 局部变量设置和获取
export interface LetState {
  get: (name: string) => any
  set: (name: string, value: any) => void
}
