interface Object {
  [key: string]: any
}
type Array = any[]

export interface Functions {
  // 文本拼接，list内部每个元素都会转为字符串进行无缝拼接，无法转换的元素默认为空字符串
  textJoin: (list: (string | number | boolean | Object | Array | undefined | null)[]) => string

  // value转为json字符串
  json: (value: string | number | boolean | Object | Array | undefined | null) => string

  /*
    * 废除
  // value转为字符串并返回，若无法转换则返回_default或空字符串
  string: (value: string | number | boolean | Object | Array | undefined | null, _default?: string) => string

  // value转为数字并返回，若无法转换则返回_default或0
  number: (value: any, _default?: number) => number

  // value转为布尔值并返回，若无法转换则返回_default或false
  boolean: (value: any, _default: boolean) => boolean

  // value转为对象并返回，若无法转换则返回_default或{}
  object: (value: any, _default?: Object) => Object

  // value转为数组并返回，若无法转换则返回_default或{}
  array: (value: any, _default?: Array) => Array

  */

  // 过滤文本首尾的空字符
  trim: (value: any) => string

  // 生成序列数组，例：[1,2,3,4,5]
  range: (start: number, stop: number, step: number) => number[]

  // 获取字符串、数组、对象的长度
  length: (value: Array | Object | string) => number

  /*
   * 废除
  // 获取数组、字符串的第1个元素
  first: (value: Array | string) => any

  // 获取数组、字符串的最后1个元素
  last: (value: Array | string) => any
  */

  // 获取指定字符在数组或字符串中位置
  indexOf: (data: Array | string, str: string) => number

  // 从数组或字符中得到指定区间的内容
  slice: (data: Array | string, start: number, end?: number) => Array | string

  /*
   *废除

  // 从数组或字符中的某个位置开始取出指定长度的内容
  part: (data: Array | string, start: number, length?: number) => Array | string

   */

  // 清空字符串、数组、对象的值，
  empty: (value: string | Array | Object) => '' | [] | {}

  // 数组按指定分隔符转为字符串
  join: (value: Array, separator: string) => string

  // 字符串按指定分隔符转为数组
  split: (value: string, separator: string) => Array

  // 数组或对象中插入元素，并返回新数组或对象
  insert: (data: Array | Object, value: any, path: (string | number)[]) => Array | Object

  // 数组或对象中插入元素，并返回新数组或对象
  delete: (value: any, path: (string | number)[]) => number

  // 判断对象或数组是否存在某个键或索引
  has: (data: Array | Object, key: number | string) => boolean

  // 过滤数组或对象中符合条件的数据
  filter: (data: Array | Object, search: (item: any, key: string | number, index: number) => boolean) => Array | Object

  // 判断数组或对象中是否至少有一个元素符合条件
  some: (data: Array | Object, search: (item: any, key: string | number, index: number) => boolean) => boolean

  // 判断数组或对象中是否每个元素都符合条件
  every: (data: Array | Object, search: (item: any, key: string | number, index: number) => boolean) => boolean

  // 将数组或对象每个元素的值进行转换
  map: (data: Array | Object, callback: (item: any) => any) => Array | Object

  // 合并数组或合并多个对象
  merge: (list: Array[] | Object[]) => Array | Object

  // 克隆数据
  clone: (
    value: string | number | boolean | Object | Array | undefined | null
  ) => string | number | boolean | Object | Array | undefined | null

  // 从数组或对象中获取指定路径的值
  path: (data: Array | Object, path: (string | number)[]) => any

  // 延时器
  wait: (callback: Function, timeout: number) => void

  // 定时器
  timer: (callback: Function, timeout: number, clear: (i: number, time: number) => boolean) => void
  /*
  * 废弃
  // 加法
  add: (left: number, right: number) => number
  */

  // 求和
  sum: (list: number[], skipNaN: boolean) => number

  // 减法
  subtract: (list: number[], skipNaN: boolean) => number

  // 乘法
  multiply: (list: number[], skipNaN: boolean) => number

  // 除法
  divide: (list: number[], skipNaN: boolean) => number

  // 取余
  mod: (left: number, right: number) => number

  /*
   * 废除
  // 转为整数
  int: (number: number) => number

  */

  // 向上保留指定小数位
  ceil: (number: number, floatLength: number) => number

  // 四舍五入指定小数位
  round: (number: number, floatLength: number) => number

  // 向下保留指定小数位
  floor: (number: number, floatLength: number) => number

  // 是否为空
  isEmpty: (value: any) => any

  // 是否不为空
  present: (value: any) => any

  // 根据条件是否成立来返回指定的结果
  expression: (expression: boolean[], operation: 'and' | 'or') => boolean

  // 根据条件是否成立来返回指定的结果
  if: (condition: () => boolean, success: () => any, fail: () => any) => any
  // 根据条件是否成立来返回指定的结果
  ifs: (rd_if: { condition: () => boolean; return: () => any }[], rd_else: { return: () => any }) => any
  // 根据条件是否成立来返回指定的结果
  when: (rd_if: { condition: () => boolean; actions: () => void }[], rd_else: { actions: () => void }) => any

  // 获取值的数据类型（string、number、boolean、array、object、null、undefined等）
  type: (data: any) => string

  // 判断值是否相等
  eq: (left: any, right: any) => boolean

  // 判断值是否不相等
  nor: (left: any, right: any) => boolean

  // 是否等于
  gt: (left: any, right: any) => boolean

  // 是否大于或等于
  gte: (left: any, right: any) => boolean

  // 是否小于
  lt: (left: any, right: any) => boolean

  // 是否小于或等于
  lte: (left: any, right: any) => boolean

  // 判断列队中是否至少有一个元素能转为true（除0、''、undefined、null、NaN都会转为true）
  or: (list: any[]) => boolean

  // 判断列队中是否所有元素都能转为true
  and: (list: any[]) => boolean

  // 判断list中是否包含search对应的值
  in: (list: any[], search: any) => boolean

  // 判断list中是否不包含search对应的值
  notin: (list: any[], search: any) => boolean

  // 检测字符串是否以特定字符串开头
  startsWith: (keyword: string, str: string) => boolean

  // 检测字符串是否以特定字符串结尾
  endsWith: (keyword: string, str: string) => boolean

  // 检测字符串是否匹配给定的正则表达式
  test: (regExp: string, str: string) => boolean
}
