import { DECLTYPE } from '../../src/const/const'
import { DATATYPE } from '../../src/const/const'
import type { Call, CallVoid } from './stmt'

type DeclType = DECLTYPE.var | DECLTYPE.expr | DECLTYPE.closure

type DeclVarValue = string | number | boolean | any[] | Record<string, any>

type DeclId = string | number
type DeclLabel = string
type DeclOtherTypes = string
type DeclOtherArgs = string[]

export interface DeclOther {
  '//'?: DeclLabel
  ':'?: DeclOtherTypes
  args?: DeclOtherArgs
}
type DeclVar = [DeclId, DECLTYPE.var, DeclVarValue, DeclOther]
type DeclExpr = [DeclId, DECLTYPE.expr, Call, DeclOther]
type DeclClosure = [DeclId, DECLTYPE.closure, Call | CallVoid, DeclOther]

// 数据库中每一行的数据结构
export type Decl = DeclVar | DeclExpr | DeclClosure

export type DeclValueType = DATATYPE.string | DATATYPE.number | DATATYPE.boolean | DATATYPE.array | DATATYPE.object
export type DeclFuncType =
  | DATATYPE.string
  | DATATYPE.number
  | DATATYPE.boolean
  | DATATYPE.array
  | DATATYPE.object
  | DATATYPE.void

export interface DeclVarApi {
  i: () => DeclId
  dt: () => DECLTYPE
  v: () => DeclVarValue
  t: () => DeclValueType[]
  l: () => DeclOther['//']
  args: () => []
}
export interface DeclExprApi {
  i: () => DeclId
  dt: () => DECLTYPE
  v: () => Call
  t: () => DeclValueType[]
  l: () => DeclOther['//']
  args: () => DeclOtherArgs
}
export interface DeclClosureApi {
  i: () => DeclId
  dt: () => DECLTYPE
  v: () => Call | CallVoid
  t: () => DeclFuncType[]
  l: () => DeclOther['//']
  args: () => DeclOtherArgs
}

type DeclGroupName = string

export type DeclGroup =
  | [DECLTYPE.var, DeclGroupName, DeclVarApi[]]
  | [DECLTYPE.expr, DeclGroupName, DeclExprApi[]]
  | [DECLTYPE.closure, DeclGroupName, DeclClosureApi[]]

export type DeclGroups = DeclGroup[]

export type GetDeclVar = (id?: DeclId) => DeclVarApi[] | DeclVarApi
export type GetDeclExpr = (id?: DeclId) => DeclExprApi[] | DeclExprApi
export type GetDeclClosure = (id?: DeclId) => DeclClosureApi[] | DeclClosureApi

export type DeclSetValue = {
  v: DeclVarValue | Call | CallVoid
  l: DeclLabel
  t: DATATYPE[]
  args?: DeclOtherArgs
}

export type DeclGetApi = (
  type: DECLTYPE,
  declId?: DeclId
) => undefined | DeclVarApi | DeclExprApi | DeclClosureApi | (DeclVarApi | DeclExprApi | DeclClosureApi)[]

export interface DeclApi {
  toDbs: (ids: DeclId[]) => Decl[]
  reload: (decl: Decl[]) => void
  get: DeclGetApi
  typeAll: (type: DECLTYPE) => (DeclVarApi | DeclExprApi | DeclClosureApi)[]
  getAll: () => (DeclVarApi | DeclExprApi | DeclClosureApi)[]
  set: (type: DECLTYPE, declId: DeclId, value: DeclSetValue) => undefined | DeclVarApi | DeclExprApi | DeclClosureApi
  add: (type: DECLTYPE, value: DeclSetValue, declId?: DeclId) => DeclVarApi | DeclExprApi | DeclClosureApi | false
  remove: (type: DECLTYPE, declId?: DeclId) => void
}
