import { Decl, DeclApi } from '../decl'
import { Hook, HooksApi } from '../hooks'
import { OldNode } from '../oldNode'
import { CmptDefineProps, CmptPropsApi } from '../cmpt/props'
import { CmptDefineSlots, CmptSlotsApi, CmptSlotApi } from '../cmpt/slots'
import { CmptDefineEvents, CmptEventsApi } from '../cmpt/events'
import { DATATYPE } from '../../const/const'

import type { NodesApi, Node, NodeApi } from '@rd/node'
import type { Css } from '@rd/css'
import type { PreviewMock } from '@rd/previewMock'
import { CmptDefineApi } from '../cmpt/index'
import { PreviewMockUser } from '@rd/previewMock'
import { expect } from 'vitest'

export interface DesignerDataData {
  tag?: string
  nodes: OldNode[]
  decl?: Decl[]
  hooks?: Hook[]
  options?: {
    props?: CmptDefineProps
    slots?: CmptDefineSlots
    events?: CmptDefineEvents
  }
  css?: Css
  mock?: PreviewMockUser[]
}
export interface DesignerData {
  _id: string
  name: string
  cat: string
  data: DesignerDataData
  thumb: string
  type: number
  state: string
  pod?: {
    id: string
    name: string
  }
  spark?: {
    id: string
    project: string
    name: string
  }
  shared: []
  preview: string
  id: string
  theme: any
  deps: any
  tags: any[]
}

export interface PageState {
  id: () => string
  module: () => string
  moduleId: () => string
  nodesApi: NodesApi
  declApi: DeclApi
  hooksApi: HooksApi
  ds: any[]
  themes: any[]
  path: []
  hooks: any[]
  bind: []
  on: []
  cat: []
  decl: []
  deps: []
  ext: []
  ui: []
  export: () => DesignerData
}

export interface PageState {
  id: () => string
  module: () => string
  moduleId: () => string
  export: () => DesignerData
}

export interface FlintState {
  id: () => string
  module: () => string
  moduleId: () => string
  export: () => DesignerData
}

export interface FlameState {
  id: () => string
  module: () => string
  moduleId: () => string
  export: () => DesignerData
}

export interface SymbolsState {
  id: () => string
  module: () => string
  moduleId: () => string
  export: () => DesignerData
}

export interface NodeActions {
  snapshot: NodeSnapshot
  isRoot: (nodeId?: string) => boolean
  remove: (nodeId?: string | null, mode: 'delete' | 'backspace' | '' = '') => void
  copy: (nodeId?: string) => void
  copyShearNode: () => { type: 'copy | shear'; nodeId: string } | undefined
  shear: (nodeId?: string) => void
  paste: (nodeId?: string) => void
  clone: (nodeId?: string) => void
  undo: () => void
  redo: () => void
  move: (nodeId?: string, toId: string, index: number | 'before' | 'after' | 'last' | 'first') => void
  moveUp: (nodeId?: string) => void
  moveDown: (nodeId?: string) => void
  moveTop: (nodeId?: string) => void
  moveBottom: (nodeId?: string) => void
  moveParent: (nodeId?: string) => void
  insert: (
    parentId: string | null,
    nodes: Node[] | OldNode[],
    index: number | null,
    newId: boolean
  ) => [NodeApi[], NodeParentId, Node[], number, NodeId[]] | undefined
  importHtml: (parentId: string | null, node: Node[]) => [NodeApi[], NodeParentId, Node[], number, NodeId[]] | undefined
  removeBind: (nodeId?: string | null, key: string) => void
  setBind: (nodeId?: string | null, key: string, value: any) => void
  getProp: (nodeId?: string | null, key: string) => any
  setProp: (nodeId?: string | null, key: string, value: any) => void
  setName: (nodeId?: string | null, value: string) => void
  setClass: (nodeId?: string | null, value: any) => void
  setStyle: (nodeId?: string | null, value: any) => void
  clearProp: (nodeId?: string | null, key: string) => void
  propOption: (nodeId?: string | null, propName: string) => { label: string; types: DATATYPE[] }
  saveflame: (nodeId?: string, remote: (nodes: OldNode, decl: Decl[]) => void) => void
  saveflint: (nodeId?: string, remote: (nodes: OldNode, decl: Decl[]) => void) => void
  getAllParents: (nodeId?: string) => NodeApi[]
  chooseLevel: (nodeId: string) => void
  get(nodeId?: string | null): NodeApi | undefined
  active: (nodeId?: NodeId) => NodeApi | undefined
  selectChild: (nodeId?: NodeId) => void
  selectParent: (nodeId?: NodeId) => void
  nodePropsOptions: (nodeId?: NodeId) => CmptDefineProps | undefined
  cmptOption: (nodeId?: NodeId) => CmptDefineApi | undefined
  slotOption: (nodeId?: NodeId) => CmptSlotApi | undefined
  slotPropOption: (nodeId?: string | null, propName: string) => { label: string; types: DATATYPE[] }
  cmptName: (nodeId?: string) => string
}

export interface NodeSnapshot {
  isFirst: () => boolean
  isLast: () => boolean
  add: (action: string, param: any, data?: any, debounceTime?: number) => void
  undo: (anode: NodesApi) => void
  redo: (anode: NodesApi) => void
}

export interface StateApi_ {
  reload: (data: DesignerData, module: string, cmptDefinesApi: CmptDefinesApi) => void
  id: () => string
  name: () => string
  cat: () => string
  data: () => DesignerDataData
  thumb: () => string
  type: () => number
  state: () => string
  module: () => string
  moduleId: () => string
  rootId: () => string
  anode: NodesApi
  decl: DeclApi
  hooks: HooksApi
  cmptProps: CmptPropsApi
  cmptSlots: CmptSlotsApi
  cmptEvents: CmptEventsApi
}

export interface StateApi extends StateApi_ {
  nodeActions: NodeActions
}
