import type { NodeOnBind, NodeOnBindApi, NodeOnName, NodeOnBindOneApi } from './nodeOnBind'
import type { NodePropsBind, NodePropsBindApi, NodePropsBindOneApi, NodePropsName } from './nodePropsBind'
import type { Call } from './stmt'

export type NodeId = string | `${string}:${NodeSlot}`
export type NodeSlot = string

export type NodeParentId = null | NodeId

export type NodeTag = string
export type NodeProps = Record<string, any>
export type NodeName = string
export type ChildrenId = string
export type NodeSlotId = string
export type NodePlugins = { [key: string]: any }
export type NodeData = {
  '='?: NodeProps | string
  '//'?: NodeName
  '@'?: [NodeOnName, Call][]
  ':'?: [NodePropsName, Call][]
  '+'?: NodePlugins
  '#'?: NodeSlotId
  '/*'?: string | undefined
  type?: string
}

// 数据库中每一行的数据结构
export type Node = [NodeId, NodeParentId, NodeTag, NodeData?]
// david新版Node节点

export interface Node3_0_data {
  comment?: string
  props?: { name: string; value: any }[]
  slotId?: string
  slot?: string
  bind?: Bind
  on?: On
  plugins?: NodePlugins
}
export interface Node3_0 {
  id?: string
  tag?: string
  type?: string
  name?: NodeName
  nodes?: Node3_0[]
  data?: Node3_0_data
}

export interface NodeApi {
  i: () => NodeId
  p: () => NodeApi | undefined // 父
  ps: () => NodeApi[] // 所有父
  pis: () => NodeId[] // 所有父id
  pi: () => NodeParentId // 父id
  t: () => NodeTag
  s: () => NodeSlotId | undefined // 当前节点如果是tpl或slot就返回插槽名称，否则undefined
  isDynamicSlot: () => boolean
  dynamicSlotRawName: () => string | undefined
  sp: () => NodeSlotId // 当前节点所在父节点的哪个插槽
  a: (name?: string) => any
  l: () => NodeName | undefined
  comment: () => string | undefined
  type: () => string | undefined
  c: () => NodeApi[]
  b: (name?: NodePropsName) => NodePropsBindOneApi | undefined
  bs: () => NodePropsBindOneApi[]
  o: (name?: NodeOnName) => NodeOnBindOneApi | undefined
  os: () => NodeOnBindOneApi[]
  pg: () => NodePlugins
  idx: () => number | undefined
  vm: (vm?: any) => any
  _: (key: string, value?: any) => any | void
  __: (key: string, value?: any) => any | void
}

export interface NodeSetApi {
  l: (value: string) => [NodeParentId, index] | undefined
  comment: (value: string) => [NodeParentId, index] | undefined
  type: (value: string) => [NodeParentId, index] | undefined
  a: (key: string, value: any) => [NodeParentId, index] | undefined
  b: (key: string, value: any) => [NodeParentId, index] | undefined
  o: (key: string, value: any) => [NodeParentId, index] | undefined
  pg: (key: string, value: any) => [NodeParentId, index] | undefined
}

export type NodeGetApi = (nodeId: NodeId) => undefined | NodeApi

export type NodeAddValue = {
  p: NodeParentId
  t: NodeTag
  a?: NodeProps
  l?: NodeName
  pg?: NodePlugins
  comment?: string
  type?: string
  // b?: [NodePropsName, Call][]
  // o?: [NodeOnName, Call][]
}
export type NodeSetValue = {
  t?: NodeTag
  a?: NodeProps
  l?: NodeName
  pg?: NodePlugins
  comment?: string
  type?: string
  // b?: [NodePropsName, Call][]
  // o?: [NodeOnName, Call][]
}

export interface NodesApi {
  // 不传nodeId表示提取所有节点，传nodeId提取指定节点及子孙节点，额外传index则返回该nodeId下指定索引的节点及子孙节点
  toDbs: (nodeId?: NodeId, index?: number | null | undefined, filter?: (NodeApi) => boolean) => Node[]
  roots: () => NodeApi[]
  nodes: () => NodeApi[]
  // props: (nodeId?: NodeId, data: string | string[] | NodeProps, value?: any) => any
  get: (nodeId?: NodeId | null) => undefined | NodeApi
  // set: (nodeId: NodeId, value: NodeSetValue) => undefined | false | NodeApi
  set: (nodeId?: NodeId | null) => NodeSetApi | undefined
  add: (value: NodeAddValue) => false | undefined | NodeApi
  clear: (nodeId?: NodeId) => void
  reload: (
    nodes: Node[],
    nodeId?: NodeId,
    before?: null | ((node: Node) => void),
    after?: (nodeApi: NodeApi) => void
  ) => void
  insert: (
    toNodeId: NodeId | null,
    nodes: Node[],
    index?: number | null,
    newId?: boolean,
    before?: null | ((node: Node) => void),
    after?: (nodeApi: NodeApi) => void
  ) => [NodeApi[], NodeParentId, Node[], number, NodeId[]] | undefined
  move: (
    nodeId: NodeId | null | undefined,
    toNodeId: NodeParentId,
    index: number | 'first' | 'last' | 'before' | 'after',
    canMove?: (toNodeApi: NodeApi) => boolean
  ) => false | [[NodeParentId, number], [NodeParentId, number]]
  _: (key: string, value?: any) => any | void
  active: (nodeId?: NodeId | null) => NodeApi | undefined
  on: NodeOnBindApi
  bind: NodePropsBindApi
  remove: (nodeId?: NodeId | null, index?: number | null) => false | [NodeParentId, number, Node[], NodeId[]]
}
